@@ -15,8 +15,8 @@ syntax highlighting of all supported filetypes.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_basestc.py 63656 2010-03-08 23:58:21Z CJP $"
-__revision__ = "$Revision: 63656 $"
+__svnid__ = "$Id: ed_basestc.py 64304 2010-05-13 16:44:35Z CJP $"
+__revision__ = "$Revision: 64304 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -654,6 +654,19 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         sel = self.GetSelection()
         return sel[0] != sel[1]
 
+    def HasMultilineSelection(self):
+        """Is the selection over multiple lines?
+        @return: bool
+
+        """
+        bMulti = False
+        sel = self.GetSelection()
+        if sel[0] != sel[1]:
+            sline = self.LineFromPosition(sel[0])
+            eline = self.LineFromPosition(sel[1])
+            bMulti = sline != eline
+        return bMulti
+
     def HidePopups(self):
         """Hide autocomp/calltip popup windows if any are active"""
         if self.AutoCompActive():
@@ -855,15 +868,6 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         """Set the buffers filename attributes from the given path"""
         self.file.SetPath(path)
 
-    def SetFocus(self):
-        """Set the focus to this control
-        @note: overridden as a hack for msw
-
-        """
-        super(EditraBaseStc, self).SetFocus()
-        if wx.Platform == '__WXMSW__':
-            wx.PostEvent(self, wx.FocusEvent(wx.wxEVT_SET_FOCUS, self.GetId()))
-
     def SetKeyWords(self, kw_lst):
         """Sets the keywords from a list of keyword sets
         @param kw_lst: [ (KWLVL, "KEWORDS"), (KWLVL2, "KEYWORDS2"), ect...]
