@@ -18,8 +18,8 @@ specific options such as commenting code.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_stc.py 62460 2009-10-21 03:13:18Z CJP $"
-__revision__ = "$Revision: 62460 $"
+__svnid__ = "$Id: ed_stc.py 64328 2010-05-16 18:39:57Z CJP $"
+__revision__ = "$Revision: 64328 $"
 
 #-------------------------------------------------------------------------#
 # Imports
@@ -547,6 +547,10 @@ class EditraStc(ed_basestc.EditraBaseStc):
 
         # If the file is different than the last save point make the backup.
         bkupmgr = ebmlib.FileBackupMgr(None, u"%s.edbkup")
+        path = _PGET('AUTOBACKUP_PATH', default=u"")
+        if path and os.path.exists(path):
+            bkupmgr.SetBackupDirectory(path)
+
         if not self._backup_done and \
            (not bkupmgr.HasBackup(fname) or bkupmgr.IsBackupNewer(fname)):
             writer = bkupmgr.GetBackupWriter(self.File)
@@ -630,14 +634,12 @@ class EditraStc(ed_basestc.EditraBaseStc):
             return
 
         elif key_code in cmpl.GetAutoCompKeys():
-            if self.AutoCompActive():
-                self.AutoCompCancel()
+            self.HidePopups()
 
-            if self.CallTipActive():
-                self.CallTipCancel()
+            uchr = unichr(key_code)
+            command = self.GetCommandStr() + uchr
+            self.PutText(uchr)
 
-            command = self.GetCommandStr() + unichr(key_code)
-            self.AddText(unichr(key_code))
             if self._config['autocomp']:
                 self.ShowAutoCompOpt(command)
 
@@ -652,8 +654,11 @@ class EditraStc(ed_basestc.EditraBaseStc):
         elif key_code in cmpl.GetCallTipKeys():
             if self.AutoCompActive():
                 self.AutoCompCancel()
+
             command = self.GetCommandStr()
-            self.AddText(unichr(key_code))
+            uchr = unichr(key_code)
+            self.PutText(uchr)
+
             if self._config['autocomp']:
                 self.ShowCallTip(command)
 
@@ -668,7 +673,12 @@ class EditraStc(ed_basestc.EditraBaseStc):
             evt.Skip()
             if self.CallTipActive():
                 self.CallTipCancel()
+#        elif key_code == wx.WXK_TAB and \
+#             True not in (evt.ControlDown(), evt.CmdDown(), 
+#                          evt.ShiftDown(), evt.AltDown()):
+#            self.Tab() # <- So action can be overridden
         else:
+#            print "IS TAB", key_code, wx.WXK_TAB
             evt.Skip()
 
     def OnKeyUp(self, evt):
@@ -841,7 +851,7 @@ class EditraStc(ed_basestc.EditraBaseStc):
                     elif evt.GetControl() or \
                         (wx.Platform == '__WXMAC__' and evt.GetAlt()):
                         # Contract all subnodes of clicked one
-                        # Note: using Alt as Ctrl can not be recieved for
+                        # Note: using Alt as Ctrl can not be received for
                         # clicks on mac (Scintilla Bug).
                         if self.GetFoldExpanded(line_clicked):
                             self.SetFoldExpanded(line_clicked, False)
@@ -927,6 +937,14 @@ class EditraStc(ed_basestc.EditraBaseStc):
                 line = line + 1
         return line
 
+    def ExpandAll(self):
+        """Expand all folded code blocks"""
+        line_count = self.GetLineCount()
+        for line_num in xrange(line_count):
+            if self.GetFoldLevel(line_num) & wx.stc.STC_FOLDLEVELHEADERFLAG:
+                if not self.GetFoldExpanded(line_num):
+                    self.Expand(line_num, True)
+
     def FindLexer(self, set_ext=u''):
         """Sets Text Controls Lexer Based on File Extension
         @param set_ext: explicit extension to use in search
@@ -995,11 +1013,8 @@ class EditraStc(ed_basestc.EditraBaseStc):
                     ed_glob.ID_ADD_BM    : self.Bookmark,
                     ed_glob.ID_DEL_ALL_BM : self.Bookmark}
 
-        if self.CallTipActive():
-            self.CallTipCancel()
-
-        if self.AutoCompActive():
-            self.AutoCompCancel()
+        # Hide autocomp popups
+        self.HidePopups()
 
         if e_obj.GetClassName() == "wxToolBar" or e_id in e_map:
             if e_id in e_map:
@@ -1430,6 +1445,10 @@ class EditraStc(ed_basestc.EditraBaseStc):
         else:
             self.LOG("[ed_stc][evt] Code Folding Turned Off")
             self._config['folding'] = False
+
+            # Ensure all code blocks have been expanded
+            self.ExpandAll()
+
             self.SetMarginWidth(ed_basestc.FOLD_MARGIN, 0)
 
     def SyntaxOnOff(self, switch=None):
@@ -1449,6 +1468,28 @@ class EditraStc(ed_basestc.EditraBaseStc):
             self.UpdateBaseStyles()
         return 0
 
+    def Tab(self):
+        """Override base method to ensure that folded blocks get unfolded
+        prior to changing the indentation.
+
+        """
+        # TODO: unfolding of folded blocks during block indent
+#        lines = list()
+#        if self.HasSelection():
+#            sel = self.GetSelection()
+#            sline = self.LineFromPosition(sel[0])
+#            eline = self.LineFromPosition(sel[1])
+#            lines = range(sline, eline+1)
+#        else:
+#            cline = self.GetCurrentLine()
+#            lines = [cline, cline+1]
+
+#        for line_num in lines:
+#            if self.GetFoldLevel(line_num) & wx.stc.STC_FOLDLEVELHEADERFLAG:
+#                if not self.GetFoldExpanded(line_num):
+#                    self.Expand(line_num, True)
+        super(EditraStc, self).Tab()
+
     def ToggleAutoIndent(self, switch=None):
         """Toggles Auto-indent On and Off
         @keyword switch: force a particular setting
